CREATE SEQUENCE IF NOT EXISTS {schema}_SEQ_QUERIES_ID START 1;

CREATE TABLE IF NOT EXISTS {schema}.queries (
    id VARCHAR PRIMARY KEY DEFAULT NEXTVAL('{schema}_SEQ_QUERIES_ID'),
    query TEXT NOT NULL,
    bm25id INT DEFAULT NULL
);
